//
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2021 - Stephane MOTTELET
//
// This file is hereby licensed under the terms of the GNU GPL v3.0,
// For more information, see the COPYING file which you should have received
//
//--------------------------------------------------------------------------

#include "IDAManager.hxx"
#include "IDABridge.hxx"

int IDARes(realtype t, N_Vector N_VectorY, N_Vector N_VectorYDot, N_Vector N_VectorRes, void *pManager)
{
    IDAManager *manager = static_cast<IDAManager *>(pManager);
    int iNbRealEq = manager->getNRealEq();

    dynlibFunPtr pFunc = manager->getEntryPointFunction(OdeManager::RES);
    dynlibFunPtr pFuncWithPar = manager->getEntryPointFunctionWithPar(OdeManager::RES);
    if (pFunc != NULL)
    {
        // Computation is made by user C/C++/Fortran dynamically lindked entrypoint
        ((IDADynRes)pFunc)(&iNbRealEq, &t, NV_DATA_S(N_VectorY), NV_DATA_S(N_VectorYDot), NV_DATA_S(N_VectorRes));
    }
    else if (pFuncWithPar != NULL)
    {
        // Computation is made by user C/C++/Fortran dynamically lindked entrypoint with double vector of parameters
        types::Double *pDblPar = manager->getParameters(OdeManager::RES)[0]->getAs<types::Double>();
        ((IDADynResWithPar)pFuncWithPar)(&iNbRealEq, &t, NV_DATA_S(N_VectorY), NV_DATA_S(N_VectorYDot), NV_DATA_S(N_VectorRes), pDblPar->get());
    }
    else
    {
        manager->computeFunction(t, NV_DATA_S(N_VectorY), NV_DATA_S(N_VectorYDot), OdeManager::RES, NV_DATA_S(N_VectorRes));
    }
    
    return 0;
}

int IDAJac(long int iNEq, realtype t, realtype c, N_Vector N_VectorY, N_Vector N_VectorYp, N_Vector N_VectorR, DlsMat DlsMat_J, void *pManager, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
{
    IDAManager *manager = static_cast<IDAManager *>(pManager);
    int iNbRealEq = manager->getNRealEq();

    dynlibFunPtr pFunc = manager->getEntryPointFunction(OdeManager::JACYYP);
    dynlibFunPtr pFuncWithPar = manager->getEntryPointFunctionWithPar(OdeManager::JACYYP);
    if (pFunc != NULL)
    {
        // Computation is made by user C/C++/Fortran dynamically lindked entrypoint
        ((IDADynJac)pFunc)(&iNbRealEq, &t, &c,NV_DATA_S(N_VectorY), NV_DATA_S(N_VectorYp), NV_DATA_S(N_VectorR), DlsMat_J->data);
    }
    else if (pFuncWithPar != NULL)
    {
        // Computation is made by user C/C++/Fortran dynamically lindked entrypoint with double vector of parameters
        types::Double *pDblPar = manager->getParameters(OdeManager::RES)[0]->getAs<types::Double>();
        ((IDADynJacWithPar)pFunc)(&iNbRealEq, &t, &c, NV_DATA_S(N_VectorY), NV_DATA_S(N_VectorYp), NV_DATA_S(N_VectorR), DlsMat_J->data, pDblPar->get());
    }
    else if (manager->getConstantFunction(OdeManager::JACY) != NULL)
    {
        // Constant dRes/dy and dRes/dyp: manually compute  dRes/dy + c*dRes/dyp
        int iOne = 1;
        int iSize;
        types::Double *pDblJacY =  manager->getConstantFunction(OdeManager::JACY);
        types::Double *pDblJacYp =  manager->getConstantFunction(OdeManager::JACYYP);
        iSize = pDblJacY->getSize();
        std::copy(pDblJacY->get(), pDblJacY->get()+iSize, DlsMat_J->data);
        C2F(daxpy)(&iSize, &c, pDblJacYp->get(), &iOne, DlsMat_J->data, &iOne);
    }
    else
    {
        manager->computeFunction(t, NV_DATA_S(N_VectorY),  NV_DATA_S(N_VectorYp), OdeManager::JACYYP, DlsMat_J->data, c);        
    }
    return 0;
}